@@ -12,10 +12,15 @@ RSpec.describe Faraday::Adapter::NetHttp do
   context 'checking http' do
     let(:url) { URI('http://example.com') }
     let(:adapter) { described_class.new }
-    let(:http) { adapter.send(:connection, url: url, request: {}) }
+    let(:ssl) { Faraday::SSLOptions.new }
+    let(:http) { adapter.send(:connection, url: url, request: {}, ssl: ssl) }
 
     it { expect(http.port).to eq(80) }
 
+    it { expect(http).not_to be_use_ssl }
+
+    it { expect(http.cert_store).to be_nil }
+
     it 'sets max_retries to 0' do
       adapter.send(:configure_request, http, {})
 
@@ -50,7 +55,12 @@ RSpec.describe Faraday::Adapter::NetHttp do
 
       it { expect(http.port).to eq(443) }
 
-      if Gem::Version.new(Faraday::VERSION) > Gem::Version.new('2.3.0')
+      it { expect(http).to be_use_ssl }
+
+      it { expect(http.cert_store).not_to be_nil }
+
+      if Gem::Version.new(Faraday::VERSION) > Gem::Version.new('2.3.0') &&
+         Gem::Version.new(RUBY_VERSION) >= Gem::Version.new('3.0.0')
         it 'supports verify_hostname option' do
           adapter.send(:configure_ssl, http, ssl)
 
@@ -130,5 +140,55 @@ RSpec.describe Faraday::Adapter::NetHttp do
 
       it { expect(response.body.encoding).to eq(::Encoding::ASCII_8BIT) }
     end
+
+    context 'when Content-Type charset is UTF-8 and space' do
+      let(:headers) { { 'Content-Type' => 'text/xml; charset= UTF-8 ' } }
+
+      it { expect(response.body.encoding).to eq(::Encoding::UTF_8) }
+    end
+  end
+
+  context 'client certificate' do
+    let(:adapter) { described_class.new }
+    let(:url) { URI('https://example.com') }
+    let(:http) { adapter.send(:connection, url: url, request: {}, ssl: ssl_options) }
+
+    before do
+      stub_request(:any, 'https://example.com')
+    end
+
+    context 'when client_cert is provided as an array' do
+      let(:cert_array) { [OpenSSL::X509::Certificate.new, OpenSSL::X509::Certificate.new] }
+      let(:ssl_options) do
+        Faraday::SSLOptions.new.tap do |ssl_options|
+          ssl_options.client_cert = cert_array
+        end
+      end
+
+      it 'sets the first cert as cert and the rest as extra_chain_cert' do
+        adapter.send(:configure_ssl, http, ssl_options)
+      end
+
+      it { expect(http.cert).to eq(cert_array.first) }
+
+      it { expect(http.extra_chain_cert).to eq(cert_array[1..]) }
+    end
+
+    context 'when client_cert is provided as a single cert' do
+      let(:cert) { OpenSSL::X509::Certificate.new }
+      let(:ssl_options) do
+        Faraday::SSLOptions.new.tap do |ssl_options|
+          ssl_options.client_cert = cert
+        end
+      end
+
+      it 'sets the cert as cert' do
+        adapter.send(:configure_ssl, http, ssl_options)
+      end
+
+      it { expect(http.cert).to eq(cert) }
+
+      it { expect(http.extra_chain_cert).to be_nil }
+    end
   end
 end
